@@ -225,9 +225,12 @@ static void sigdelread_dsp(t_sigdelread *x, t_signal **sp)
         sigdelread_float(x, x->x_deltime);
         dsp_add(sigdelread_perform, 4,
             sp[0]->s_vec, &delwriter->x_cspace, &x->x_delsamps, sp[0]->s_n);
+        /* check block size - but only if delwriter has been initialized */
+        if (delwriter->x_cspace.c_n > 0 && sp[0]->s_n > delwriter->x_cspace.c_n)
+            pd_error(x, "delread~ %s: blocksize larger than delwrite~ buffer", x->x_sym->s_name);
     }
     else if (*x->x_sym->s_name)
-        error("delread~: %s: no such delwrite~",x->x_sym->s_name);
+        pd_error(x, "delread~: %s: no such delwrite~",x->x_sym->s_name);
 }
 
 static void sigdelread_setup(void)
@@ -277,6 +280,12 @@ static t_int *sigvd_perform(t_int *w)
     t_sample fn = n-1;
     t_sample *vp = ctl->c_vec, *bp, *wp = vp + ctl->c_phase;
     t_sample zerodel = x->x_zerodel;
+    if (limit < 0) /* blocksize is larger than delread~ buffer size */
+    {
+        while (n--)
+            *out++ = 0;
+        return (w+6);
+    }
     while (n--)
     {
         t_sample delsamps = x->x_sr * *in++ - zerodel, frac;
@@ -291,7 +300,7 @@ static t_int *sigvd_perform(t_int *w)
         idelsamps = delsamps;
         frac = delsamps - (t_sample)idelsamps;
         bp = wp - idelsamps;
-        if (bp < vp + 4) bp += nsamps;
+        if (bp < vp + XTRASAMPS) bp += nsamps;
         d = bp[-3];
         c = bp[-2];
         b = bp[-1];
@@ -319,9 +328,12 @@ static void sigvd_dsp(t_sigvd *x, t_signal **sp)
         dsp_add(sigvd_perform, 5,
             sp[0]->s_vec, sp[1]->s_vec,
                 &delwriter->x_cspace, x, sp[0]->s_n);
+        /* check block size - but only if delwriter has been initialized */
+        if (delwriter->x_cspace.c_n > 0 && sp[0]->s_n > delwriter->x_cspace.c_n)
+            pd_error(x, "vd~ %s: blocksize larger than delwrite~ buffer", x->x_sym->s_name);
     }
     else if (*x->x_sym->s_name)
-        error("vd~: %s: no such delwrite~",x->x_sym->s_name);
+        pd_error(x, "vd~: %s: no such delwrite~",x->x_sym->s_name);
 }
 
 static void sigvd_setup(void)
