@@ -11,6 +11,15 @@ and also link in the fftw library.  You can only have one of these three
 linked in.  The configure script can be used to select which one.
 */
 
+/* ------------------ initialization and cleanup -------------------------- */
+void mayer_init( void);
+void mayer_term( void);
+
+static void fftclass_cleanup(t_class *c)
+{
+    mayer_term();
+}
+
 /* ---------------- utility functions for DSP chains ---------------------- */
 
     /* swap two arrays */
@@ -128,6 +137,7 @@ static void sigfft_setup(void)
     CLASS_MAINSIGNALIN(sigfft_class, t_sigfft, x_f);
     class_addmethod(sigfft_class, (t_method)sigfft_dsp,
         gensym("dsp"), A_CANT, 0);
+    mayer_init();
 
     sigifft_class = class_new(gensym("ifft~"), sigifft_new, 0,
         sizeof(t_sigfft), 0, 0);
@@ -135,6 +145,7 @@ static void sigfft_setup(void)
     class_addmethod(sigifft_class, (t_method)sigifft_dsp,
         gensym("dsp"), A_CANT, 0);
     class_sethelpsymbol(sigifft_class, gensym("fft~"));
+    mayer_init();
 }
 
 /* ----------------------- rfft~ -------------------------------- */
@@ -193,6 +204,7 @@ static void sigrfft_setup(void)
     class_addmethod(sigrfft_class, (t_method)sigrfft_dsp,
         gensym("dsp"), A_CANT, 0);
     class_sethelpsymbol(sigrfft_class, gensym("fft~"));
+    mayer_init();
 }
 
 /* ----------------------- rifft~ -------------------------------- */
@@ -254,6 +266,7 @@ static void sigrifft_setup(void)
     class_addmethod(sigrifft_class, (t_method)sigrifft_dsp,
         gensym("dsp"), A_CANT, 0);
     class_sethelpsymbol(sigrifft_class, gensym("fft~"));
+    mayer_init();
 }
 
 /* ----------------------- framp~ -------------------------------- */
@@ -342,6 +355,7 @@ static void sigframp_setup(void)
     CLASS_MAINSIGNALIN(sigframp_class, t_sigframp, x_f);
     class_addmethod(sigframp_class, (t_method)sigframp_dsp,
         gensym("dsp"), A_CANT, 0);
+    mayer_init();
 }
 
 /* ------------------------ global setup routine ------------------------- */
