@@ -153,7 +153,8 @@ int iemgui_modulo_color(int col)
 
 t_symbol *iemgui_dollar2raute(t_symbol *s)
 {
-    char buf[MAXPDSTRING+1], *s1, *s2;
+    const char *s1;
+    char buf[MAXPDSTRING+1], *s2;
     if (strlen(s->s_name) >= MAXPDSTRING)
         return (s);
     for (s1 = s->s_name, s2 = buf; ; s1++, s2++)
@@ -168,7 +169,8 @@ t_symbol *iemgui_dollar2raute(t_symbol *s)
 
 t_symbol *iemgui_raute2dollar(t_symbol *s)
 {
-    char buf[MAXPDSTRING+1], *s1, *s2;
+    const char *s1;
+    char buf[MAXPDSTRING+1], *s2;
     if (strlen(s->s_name) >= MAXPDSTRING)
         return (s);
     for (s1 = s->s_name, s2 = buf; ; s1++, s2++)
