@@ -531,7 +531,7 @@ done:
 
 /* ------ traversal routines for code that can't see our structures ------ */
 
-int obj_noutlets(t_object *x)
+int obj_noutlets(const t_object *x)
 {
     int n;
     t_outlet *o;
@@ -539,7 +539,7 @@ int obj_noutlets(t_object *x)
     return (n);
 }
 
-int obj_ninlets(t_object *x)
+int obj_ninlets(const t_object *x)
 {
     int n;
     t_inlet *i;
@@ -548,7 +548,7 @@ int obj_ninlets(t_object *x)
     return (n);
 }
 
-t_outconnect *obj_starttraverseoutlet(t_object *x, t_outlet **op, int nout)
+t_outconnect *obj_starttraverseoutlet(const t_object *x, t_outlet **op, int nout)
 {
     t_outlet *o = x->ob_outlet;
     while (nout-- && o) o = o->o_next;
@@ -622,7 +622,7 @@ void obj_moveoutletfirst(t_object *x, t_outlet *o)
     /* routines for DSP sorting, which are used in d_ugen.c and g_canvas.c */
     /* LATER try to consolidate all the slightly different routines. */
 
-int obj_nsiginlets(t_object *x)
+int obj_nsiginlets(const t_object *x)
 {
     int n;
     t_inlet *i;
@@ -633,7 +633,7 @@ int obj_nsiginlets(t_object *x)
 }
 
     /* get the index, among signal inlets, of the mth inlet overall */
-int obj_siginletindex(t_object *x, int m)
+int obj_siginletindex(const t_object *x, int m)
 {
     int n = 0;
     t_inlet *i;
@@ -653,7 +653,7 @@ int obj_siginletindex(t_object *x, int m)
     return (-1);
 }
 
-int obj_issignalinlet(t_object *x, int m)
+int obj_issignalinlet(const t_object *x, int m)
 {
     t_inlet *i;
     if (x->ob_pd->c_firstin)
@@ -667,7 +667,7 @@ int obj_issignalinlet(t_object *x, int m)
     return (i && (i->i_symfrom == &s_signal));
 }
 
-int obj_nsigoutlets(t_object *x)
+int obj_nsigoutlets(const t_object *x)
 {
     int n;
     t_outlet *o;
@@ -676,7 +676,7 @@ int obj_nsigoutlets(t_object *x)
     return (n);
 }
 
-int obj_sigoutletindex(t_object *x, int m)
+int obj_sigoutletindex(const t_object *x, int m)
 {
     int n;
     t_outlet *o2;
@@ -689,7 +689,7 @@ int obj_sigoutletindex(t_object *x, int m)
     return (-1);
 }
 
-int obj_issignaloutlet(t_object *x, int m)
+int obj_issignaloutlet(const t_object *x, int m)
 {
     int n;
     t_outlet *o2;
@@ -697,7 +697,7 @@ int obj_issignaloutlet(t_object *x, int m)
     return (o2 && (o2->o_sym == &s_signal));
 }
 
-t_float *obj_findsignalscalar(t_object *x, int m)
+t_float *obj_findsignalscalar(const t_object *x, int m)
 {
     t_inlet *i;
     if (x->ob_pd->c_firstin && x->ob_pd->c_floatsignalin)
@@ -737,7 +737,7 @@ int outlet_getsignalindex(t_outlet *x)
     return (n);
 }
 
-void obj_saveformat(t_object *x, t_binbuf *bb)
+void obj_saveformat(const t_object *x, t_binbuf *bb)
 {
     if (x->te_width)
         binbuf_addv(bb, "ssf;", &s__X, gensym("f"), (float)x->te_width);
