@@ -222,10 +222,10 @@ void namelist_free(t_namelist *listwas)
     }
 }
 
-char *namelist_get(t_namelist *namelist, int n)
+const char *namelist_get(const t_namelist *namelist, int n)
 {
     int i;
-    t_namelist *nl;
+    const t_namelist *nl;
     for (i = 0, nl = namelist; i < n && nl; i++, nl = nl->nl_next)
         ;
     return (nl ? nl->nl_string : 0);
@@ -379,7 +379,11 @@ static int do_open_via_path(const char *dir, const char *name,
         if ((fd = sys_trytoopenone(nl->nl_string, name, ext,
             dirresult, nameresult, size, bin)) >= 0)
                 return (fd);
-
+        /* next go through the temp paths from the commandline */
+    for (nl = STUFF->st_temppath; nl; nl = nl->nl_next)
+        if ((fd = sys_trytoopenone(nl->nl_string, name, ext,
+            dirresult, nameresult, size, bin)) >= 0)
+                return (fd);
         /* next look in built-in paths like "extra" */
     if (sys_usestdpath)
         for (nl = STUFF->st_staticpath; nl; nl = nl->nl_next)
@@ -573,7 +577,8 @@ void sys_doflags( void)
     dollars, and semis down here. */
 t_symbol *sys_decodedialog(t_symbol *s)
 {
-    char buf[MAXPDSTRING], *sp = s->s_name;
+    char buf[MAXPDSTRING];
+    const char *sp = s->s_name;
     int i;
     if (*sp != '+')
         bug("sys_decodedialog: %s", sp);
@@ -614,6 +619,18 @@ void sys_set_searchpath( void)
     sys_gui("set ::sys_searchpath $::tmp_path\n");
 }
 
+    /* send the temp paths from the commandline to pd-gui */
+void sys_set_temppath(void)
+{
+    int i;
+    t_namelist *nl;
+
+    sys_gui("set ::tmp_path {}\n");
+    for (nl = STUFF->st_temppath, i = 0; nl; nl = nl->nl_next, i++)
+        sys_vgui("lappend ::tmp_path {%s}\n", nl->nl_string);
+    sys_gui("set ::sys_temppath $::tmp_path\n");
+}
+
     /* send the hard-coded search path to pd-gui */
 void sys_set_extrapath( void)
 {
