@@ -85,7 +85,7 @@ static void *table_new(t_symbol *s, t_floatarg f)
 }
 
     /* return true if the "canvas" object is a "table". */
-int canvas_istable(t_canvas *x)
+int canvas_istable(const t_canvas *x)
 {
     t_atom *argv = (x->gl_obj.te_binbuf? binbuf_getvec(x->gl_obj.te_binbuf):0);
     int argc = (x->gl_obj.te_binbuf? binbuf_getnatom(x->gl_obj.te_binbuf) : 0);
@@ -175,6 +175,7 @@ static void *array_define_new(t_symbol *s, int argc, t_atom *argv)
         garray are being created.  There may be other, unknown side effects. */
     x->gl_obj.ob_pd = array_define_class;
     array_define_yrange(x, ylo, yhi);
+    outlet_new(&x->gl_obj, &s_pointer);
     return (x);
 }
 
@@ -216,7 +217,22 @@ static void array_define_send(t_glist *x, t_symbol *s)
         pd_pointer(s->s_thing, &gp);
         gpointer_unset(&gp);
     }
-    else bug("array_define_anything");
+    else bug("array_define_send");
+}
+
+static void array_define_bang(t_glist *x)
+{
+    t_glist *gl = (x->gl_list ? pd_checkglist(&x->gl_list->g_pd) : 0);
+    if (gl && gl->gl_list && pd_class(&gl->gl_list->g_pd) == garray_class)
+    {
+        t_gpointer gp;
+        gpointer_init(&gp);
+        gpointer_setglist(&gp, gl,
+            garray_getscalar((t_garray *)gl->gl_list));
+        outlet_pointer(x->gl_obj.ob_outlet, &gp);
+        gpointer_unset(&gp);
+    }
+    else bug("array_define_bang");
 }
 
     /* just forward any messages to the garray */
@@ -817,7 +833,7 @@ static void *arrayobj_new(t_symbol *s, int argc, t_atom *argv)
         pd_this->pd_newest = array_define_new(s, argc, argv);
     else
     {
-        char *str = argv[0].a_w.w_symbol->s_name;
+        const char *str = argv[0].a_w.w_symbol->s_name;
         if (!strcmp(str, "d") || !strcmp(str, "define"))
             pd_this->pd_newest = array_define_new(s, argc-1, argv+1);
         else if (!strcmp(str, "size"))
@@ -856,6 +872,7 @@ void x_array_setup(void )
     canvas_add_for_class(array_define_class);
     class_addmethod(array_define_class, (t_method)array_define_send,
         gensym("send"), A_SYMBOL, 0);
+    class_addbang(array_define_class, array_define_bang);
     class_addanything(array_define_class, array_define_anything);
     class_sethelpsymbol(array_define_class, gensym("array-object"));
     class_setsavefn(array_define_class, array_define_save);
