@@ -95,7 +95,7 @@ static void pdfloat_float(t_pdfloat *x, t_float f)
 }
 
 #ifdef _MSC_VER
-#define strtof _atoldbl
+#define strtof(a,b) _atoldbl(a,*b)
 #endif
 
 static void pdfloat_symbol(t_pdfloat *x, t_symbol *s)
@@ -1257,7 +1257,7 @@ typedef struct _makefilename
 } t_makefilename;
 
 static const char* _formatscan(const char*str, t_printtype*typ) {
-    int num=0, infmt=0;
+    int infmt=0;
     for (; *str; str++) {
         if (!infmt && *str=='%') {
             infmt=1;
@@ -1294,9 +1294,7 @@ static const char* _formatscan(const char*str, t_printtype*typ) {
 
 static void makefilename_scanformat(t_makefilename *x)
 {
-    int num=0, infmt=0;
     const char *str;
-    char *chr;
     t_printtype typ;
     if (!x->x_format) return;
     str = x->x_format->s_name;
@@ -1334,7 +1332,7 @@ static void makefilename_float(t_makefilename *x, t_floatarg f)
     }
     switch(x->x_accept) {
     case NONE:
-        sprintf(buf, x->x_format->s_name);
+        sprintf(buf, "%s",  x->x_format->s_name);
         break;
     case INT: case POINTER:
         sprintf(buf, x->x_format->s_name, (int)f);
@@ -1373,7 +1371,7 @@ static void makefilename_symbol(t_makefilename *x, t_symbol *s)
         sprintf(buf, x->x_format->s_name, 0.);
         break;
     case NONE:
-        sprintf(buf, x->x_format->s_name);
+        sprintf(buf, "%s", x->x_format->s_name);
         break;
     default:
         sprintf(buf, "%s", x->x_format->s_name);
@@ -1397,7 +1395,7 @@ static void makefilename_bang(t_makefilename *x)
         sprintf(buf, x->x_format->s_name, 0.);
         break;
     case NONE:
-        sprintf(buf, x->x_format->s_name);
+        sprintf(buf, "%s", x->x_format->s_name);
         break;
     default:
         sprintf(buf, "%s", x->x_format->s_name);
