@@ -203,7 +203,7 @@ static void *openpanel_new( void)
 
 static void openpanel_symbol(t_openpanel *x, t_symbol *s)
 {
-    char *path = (s && s->s_name) ? s->s_name : "\"\"";
+    const char *path = (s && s->s_name) ? s->s_name : "\"\"";
     sys_vgui("pdtk_openpanel {%s} {%s}\n", x->x_s->s_name, path);
 }
 
@@ -259,7 +259,7 @@ static void *savepanel_new( void)
 
 static void savepanel_symbol(t_savepanel *x, t_symbol *s)
 {
-    char *path = (s && s->s_name) ? s->s_name : "\"\"";
+    const char *path = (s && s->s_name) ? s->s_name : "\"\"";
     sys_vgui("pdtk_savepanel {%s} {%s}\n", x->x_s->s_name, path);
 }
 
